#include "Stage.h"
#include "Stages.h"
#include "Main.h"
#include "Vram.h"
#include "Chars.h"
#include "CopyMemory.h"
#include "Fighter.h"
#include "FighterBullet.h"
#include "SkyEnemy.h"
#include "EnemyBullet.h"
#include "Bang.h"
#include "Math.h"
#include "Status.h"
#include "Barrier.h"
#include "GroundEnemy.h"
#include "Fort.h"
#include "Sprite.h"
#include "Item.h"
#include "Ground.h"

constexpr byte TerrainCharCount = (Char_End - Char_Terrain) / 2;
constexpr byte ForestChar = Char_Terrain + 3;
constexpr byte FortStartY = TileSize;

static ptr<Stage> pStage;
static ptr<byte> pMap;
static byte skyElementCount;
ptr<SkyElement> pSkyElement;

static byte mapOffset;


static void FillTiles()
{
    byte row, column;
    ptr<byte> pSource, pGround;

    row = (topRow - 1) >> 2;

    pSource = pMap;
    pGround = Ground;
    for (column = 0; column < VramWidth / TileSize; ++column) {
        ptr<Tile> pTile;
        ptr<byte> pSourceChar;
        
        pTile = &Tiles[*pSource];
        ++pSource;
        pSourceChar = pTile->chars;
        repeat (4) {
            repeat (4) {
                *pGround = *pSourceChar;
                ++pSourceChar;
                ++pGround;
            }
            pGround += StageWidth - 4;
        }
        pGround += 4 - StageWidth * 4;
    }
}


void InitStage() 
{
    byte i, j;
    i = 0;
    j = 0;
    pStage = Stages;
    while (i < CurrentStage) {
        ++pStage;
        ++i;
        ++j;
        if (j >= StageCount) {
            pStage = Stages;
            j = 0;
        }
    }

    mapOffset = pStage->mapOffset;
    skyElementCount = pStage->skyElementCount;
    pSkyElement = pStage->pSkyElements;
    pMap = MapBytes + (MapHeight - 1) * MapWidth + mapOffset;
    topRow = MapHeight * TileSize;
    groundElementCount = pStage->groundElementCount;
    pGroundElement = pStage->pGroundElement;

    dotOffset = 0;

    UpdateTerrainChars();
    FillTiles();
    GroundChanged = true;
}

void InitTrying()
{
    SkyElementBits = 0;
    RndIndex = 0;

    HideAllSprites();
    InitFighter();
    InitFighterBullets();
    InitSkyEnemies();
    InitBarriers();
    InitGroundEnemies();
    InitFort();
    InitEnemyBullets();
    InitBangs();
    InitItem();

    FillMemory(Ground + StageWidth * 4, StageWidth, ForestChar - 1);
    FillMemory(Ground + StageWidth * 5, StageWidth * (GroundHeight - 4 - 1), ForestChar);
}


// void _deb(){}
void ScrollGround()
{
    if (FortLife != 0) return;

    dotOffset += (8 / CoordRate);
    if (dotOffset == 8) {
        dotOffset = 0;
        UpdateTerrainChars();
        RollDown();
        --topRow;
        if (topRow == FortStartY && FortLife == 0) {
            SwitchChars();
            UpdateTerrainChars();
            StartFort();
        }        
        if (topRow == 0) {
            ++CurrentStage;
            InitStage();
            PrintStage();
        }
        else if ((topRow & 3) == 0) {
            pMap -= MapWidth;
                                // _deb();
            FillTiles();
        }
        if (skyElementCount > 0 && pSkyElement->row >= topRow) {
            SkyElementBits = pSkyElement->bits;
            ++pSkyElement;
            --skyElementCount;
        }
        GroundChanged = true;
        StartSkyEnemy();
        StartGroundEnemy();
    }
    else {
        UpdateTerrainChars();
        SwitchChars();
    }
    ScrollGroundEnemy();
}
