#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Chars.h"
#include "Print.h"
#include "CopyMemory.h"
#include "Stage.h"
#include "Ground.h"

constexpr byte StatusHeight = 2;

word Score;
word HiScore;
byte RemainCount;
byte CurrentStage;
byte[StageWidth * StatusHeight] Status;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

static void PrintRemain(ptr<byte> pStatus)
{
    byte c;
    c = Char_Remain;
    repeat (2) {
        repeat (2) {
            *pStatus = c;
            ++c;
            ++pStatus;
        }
        pStatus += StageWidth - 2;
    }
}

void PrintStatus() 
{
    FillMemory(Status, StageWidth * StatusHeight, Char_Space);
    PrintS(Status + 1, "SCORE");
    PrintS(Status + 8, "HI-SCORE");
    PrintS(Status + 18, "STAGE");

    {
        ptr<byte> pStatus;
        pStatus = Status + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                PrintRemain(pStatus);
                Status[29] = Char_Space;
                Status[30] = Char_Space;
                PrintC(Status + StageWidth + 29, i + '0');
                Status[StageWidth + 30] = Char_Space;
            }
            else {
                do {
                    PrintRemain(pStatus);
                    pStatus += 2;
                    --i;
                } while (i > 0);
            }
        }
    }

    PrintScore();
    PrintStage();
}

void PrintScore()
{
    ptr<byte> pStatus;
    
    pStatus = PrintNumber5(Status + StageWidth, Score);
    PrintC(pStatus, '0');

    pStatus = PrintNumber5(Status + StageWidth + 9, HiScore);
    PrintC(pStatus, '0');
}

void PrintStage()
{
    PrintByteNumber3(Status + StageWidth + 20, CurrentStage + 1);
}

void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	I
        0x1f, 0x28, 0x22, 0x1c, 
        0x1b, 0x27, 0x1e, 0x1b, 
        0x1b, 0x27, 0x1e, 0x1b, 
        0x1f, 0x20, 0x20, 0x1c, 
        //	M
        0x27, 0x22, 0x22, 0x26, 
        0x27, 0x1e, 0x1e, 0x2a, 
        0x27, 0x1e, 0x1e, 0x2a, 
        0x1f, 0x1c, 0x1c, 0x20, 
        //	P
        0x27, 0x22, 0x20, 0x26, 
        0x27, 0x1e, 0x1b, 0x2a, 
        0x27, 0x22, 0x20, 0x1c, 
        0x1f, 0x1c, 0x1b, 0x1b, 
        //	E
        0x27, 0x22, 0x20, 0x1c, 
        0x27, 0x26, 0x25, 0x1d, 
        0x27, 0x1e, 0x1b, 0x1b, 
        0x1f, 0x20, 0x20, 0x1c, 
        //	T
        0x1f, 0x28, 0x22, 0x1c, 
        0x1b, 0x27, 0x1e, 0x1b, 
        0x1b, 0x27, 0x1e, 0x1b, 
        0x1b, 0x1f, 0x1c, 0x1b, 
        //	U
        0x27, 0x1e, 0x1b, 0x2a, 
        0x27, 0x1e, 0x1b, 0x2a, 
        0x27, 0x1e, 0x1b, 0x2a, 
        0x1b, 0x20, 0x20, 0x1c, 
        //	S
        0x23, 0x22, 0x20, 0x26, 
        0x1f, 0x26, 0x25, 0x1d, 
        0x23, 0x1d, 0x1b, 0x2a, 
        0x1b, 0x20, 0x20, 0x1c, 
    };

    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    StatusToVram();
    SwitchVram();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (32 - 4 * 7) / 2;
        p = TitleBytes;
        repeat (7) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 7, "PUSH TRIGGER BUTTON");
    // {
    //     byte c;
    //     ptr<byte> pVram;
    //     pVram = Vram+32*10;
    //     c = 0;
    //     repeat (0) {
    //         pVram = Put(pVram, c);
    //         ++c;
    //     }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
    // StatusToVram();
}
